/*
 * Copyright (C) 2020 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
**/
#include "logind.h"
#include <QCoreApplication>
#include <QDebug>
#include <QDBusConnection>
#include <QDBusInterface>
#include <QDBusConnectionInterface>

const static QString login1Service = QStringLiteral("org.freedesktop.login1");
const static QString login1Path = QStringLiteral("/org/freedesktop/login1");
const static QString propertiesInterface = QStringLiteral("org.freedesktop.DBus.Properties");
const static QString login1ManagerInterface = QStringLiteral("org.freedesktop.login1.Manager");
const static QString login1SessionInterface = QStringLiteral("org.freedesktop.login1.Session");

LogindIntegration::LogindIntegration(QObject *parent)
    : QObject(parent)
{
    QDBusInterface loginInterface(login1Service,
                             login1Path,
                             login1ManagerInterface,
                             QDBusConnection::systemBus());
    QDBusReply<QDBusObjectPath> sessionPath = loginInterface.call("GetSession", "auto");
    if(!sessionPath.isValid()){
        qWarning()<< "Get session error:" << sessionPath.error();
    }
    else{
        QString session = sessionPath.value().path();
        QDBusConnection::systemBus().connect(login1Service,
                      session,
                      login1SessionInterface,
                      QStringLiteral("Lock"),
                      this,
                      SIGNAL(requestLock()));
        QDBusConnection::systemBus().connect(login1Service,
                      session,
                      login1SessionInterface,
                      QStringLiteral("Unlock"),
                      this,
                      SIGNAL(requestUnlock()));

        // 获取会话激活状态
        QDBusInterface iface(login1Service,
                             session,
                             propertiesInterface,
                             QDBusConnection::systemBus());
        QDBusReply<QVariant> reply = iface.call("Get", login1SessionInterface, "Active");
        if (reply.isValid()) {
            m_isSessionActive = reply.value().toBool();
            qDebug()<<"Session is active:"<<m_isSessionActive;
        } else {
            qDebug() << "Failed to get session active state!";
        }

        // 监听属性变化
        QDBusConnection::systemBus().connect(login1Service,
                                             session,
                                             propertiesInterface,
                                             "PropertiesChanged",
                                             this,
                                             SLOT(onSessionPropChanged(QString, QVariantMap, QStringList)));
    }
    return;
}

LogindIntegration::~LogindIntegration()
{

}

void LogindIntegration::onSessionPropChanged(QString strInterface, QVariantMap mapVar, QStringList listValue)
{
    if (login1SessionInterface == strInterface) {
        qDebug()<<"onSessionPropChanged:"<<strInterface<<mapVar<<listValue;
        QVariantMap::iterator itVar = mapVar.begin();
        for ( ; itVar != mapVar.end(); itVar++) {
            if (itVar.key() == "Active") {
                QVariant varValue = itVar.value();
                m_isSessionActive = varValue.toBool();
                if (!m_isSessionActive) {
                    Q_EMIT requestLock();
                }
                Q_EMIT sessionActiveChanged(m_isSessionActive);
                break;
            }
        }
    }
}

bool LogindIntegration::isSessionActive()
{
    return m_isSessionActive;
}
